"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PiAPITaskStatus = void 0;
const GenericFunctions_1 = require("../shared/GenericFunctions");
function ensureObject(value) {
    if (value !== null && typeof value === 'object') {
        return value;
    }
    return {};
}
class PiAPITaskStatus {
    constructor() {
        this.description = {
            displayName: 'PiAPI Task Status',
            name: 'piAPITaskStatus',
            icon: 'file:../piapi.svg',
            group: ['transform'],
            version: 1,
            description: 'Check the status of a PiAPI task',
            defaults: {
                name: 'PiAPI Task Status',
            },
            inputs: ["main"],
            outputs: ["main"],
            credentials: [
                {
                    name: 'piAPIApi',
                    required: true,
                },
            ],
            properties: [
                {
                    displayName: 'Task ID',
                    name: 'taskId',
                    type: 'string',
                    default: '',
                    required: true,
                    description: 'The ID of the task to check',
                },
                {
                    displayName: 'Return Only Media URL',
                    name: 'returnOnlyImageUrl',
                    type: 'boolean',
                    default: false,
                    description: 'Whether to return only the image/video URL instead of the full task data',
                },
                {
                    displayName: 'Return Binary Data',
                    name: 'returnBinaryData',
                    type: 'boolean',
                    default: false,
                    description: 'Whether to return the Image/Video as binary data',
                },
                {
                    displayName: 'Binary Property',
                    name: 'binaryPropertyName',
                    type: 'string',
                    default: 'data',
                    required: true,
                    displayOptions: {
                        show: {
                            returnBinaryData: [true],
                        },
                    },
                    description: 'Name of the binary property to which to write the data of the image',
                },
                {
                    displayName: 'Normalize Response',
                    name: 'normalizeResponse',
                    type: 'boolean',
                    default: true,
                    description: 'Whether to normalize the response to include both original input data and task data',
                },
            ],
        };
    }
    async execute() {
        var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l;
        const items = this.getInputData();
        const returnData = [];
        for (let i = 0; i < items.length; i++) {
            const taskId = this.getNodeParameter('taskId', i);
            if (!taskId || taskId.trim() === '') {
                throw new Error('Task ID is required and cannot be empty');
            }
            const returnOnlyImageUrl = this.getNodeParameter('returnOnlyImageUrl', i, false);
            const returnBinaryData = this.getNodeParameter('returnBinaryData', i, false);
            const normalizeResponse = this.getNodeParameter('normalizeResponse', i, true);
            const binaryPropertyName = returnBinaryData
                ? this.getNodeParameter('binaryPropertyName', i)
                : '';
            try {
                const encodedTaskId = encodeURIComponent(taskId.trim());
                const response = await GenericFunctions_1.piApiRequest.call(this, 'GET', `/api/v1/task/${encodedTaskId}`);
                if (!response.data || response.code !== 200) {
                    throw new Error(`Failed to retrieve task: ${response.message || 'Unknown error'}`);
                }
                const mediaUrl = ((_b = (_a = response.data) === null || _a === void 0 ? void 0 : _a.output) === null || _b === void 0 ? void 0 : _b.image_url) || ((_d = (_c = response.data) === null || _c === void 0 ? void 0 : _c.output) === null || _d === void 0 ? void 0 : _d.video_url);
                if (returnOnlyImageUrl && mediaUrl) {
                    const responseData = normalizeResponse
                        ? {
                            ...items[i].json,
                            data: {
                                ...ensureObject((_e = items[i].json) === null || _e === void 0 ? void 0 : _e.data),
                                mediaUrl,
                                type: ((_g = (_f = response.data) === null || _f === void 0 ? void 0 : _f.output) === null || _g === void 0 ? void 0 : _g.image_url) ? 'image' : 'video',
                            }
                        }
                        : {
                            mediaUrl,
                            type: ((_j = (_h = response.data) === null || _h === void 0 ? void 0 : _h.output) === null || _j === void 0 ? void 0 : _j.image_url) ? 'image' : 'video',
                        };
                    returnData.push({ json: responseData });
                }
                else if (returnBinaryData && mediaUrl) {
                    const imageUrl = mediaUrl;
                    const imageData = await this.helpers.request({
                        method: 'GET',
                        url: imageUrl,
                        encoding: null,
                        resolveWithFullResponse: true,
                    });
                    const newItem = {
                        json: normalizeResponse
                            ? {
                                ...items[i].json,
                                data: {
                                    ...ensureObject((_k = items[i].json) === null || _k === void 0 ? void 0 : _k.data),
                                    task: response.data,
                                }
                            }
                            : response.data,
                        binary: {},
                    };
                    if (newItem.binary) {
                        newItem.binary[binaryPropertyName] = await this.helpers.prepareBinaryData(Buffer.from(imageData.body), imageUrl.split('/').pop() || 'image.png', imageData.headers['content-type']);
                    }
                    returnData.push(newItem);
                }
                else {
                    const responseData = normalizeResponse
                        ? {
                            ...items[i].json,
                            data: {
                                ...ensureObject((_l = items[i].json) === null || _l === void 0 ? void 0 : _l.data),
                                task: response.data,
                            }
                        }
                        : response.data;
                    returnData.push({ json: responseData });
                }
            }
            catch (error) {
                if (this.continueOnFail()) {
                    returnData.push({
                        json: {
                            error: error.message,
                        },
                    });
                    continue;
                }
                throw error;
            }
        }
        return [returnData];
    }
}
exports.PiAPITaskStatus = PiAPITaskStatus;
//# sourceMappingURL=PiAPITaskStatus.node.js.map